import click
from collections import OrderedDict
from jinja2 import Environment, PackageLoader
from merle.fetched_resource import FetchedResource


DEFAULT_TEMPLATE = 'yaml.jinja2.txt'

def dumper(obj, template_name=DEFAULT_TEMPLATE):
    env = Environment(loader=PackageLoader('merle', 'templates'),
                      trim_blocks=True, lstrip_blocks=True)
    t = env.get_template(template_name)
    return t.render(f=obj)


@click.command()
@click.argument('url')
def fetch_metadata(url):
    o = OrderedDict()
    f = FetchedResource(url)
    o['slug'] = f.slug
    o['url'] = f.returned_url
    o['title'] = f.title
    o['description'] = f.description
    o['fetched_at'] = f.fetched_at
    o['published_at'] = f.published_at
    o['authors'] = f.authors
    o['word_count'] = f.word_count
    o['excerpt'] = f.excerpt


    click.echo(dumper(o))



if __name__ == '__main__':
    fetch_metadata()



# import requests
from merle.extractor import extract_element_from_doc
from datetime import datetime
from slugify import slugify_url
from urllib.parse import urlparse
from newspaper import fulltext, Article
import re

EXCERPT_WORD_COUNT = 60

class FetchedResource:
    def __init__(self, url):
        self.fetched_url = url
        self.fetched_at = datetime.now()
        # need to better use newspaper egg
        self.article = self._articlize() # this needs to be killed
        self.returned_url = self.article.url

        self.html = self.article.html
        self.fulltext = re.sub('\s+', ' ', fulltext(self.html)).strip()
        # resp = self._fetch(self.fetched_url)
        self.doc = self.article.doc
        # self.headers = resp.headers
        # self.encoding = resp.encoding
        # self.status_code = resp.status_code
        # self.history = resp.history

        # elements
        self.canonical_url = self._extract_element('canonical_url')[0] or self.returned_url

        self.titles = self._extract_element('title')
        self.title = self.titles[0]
        self.descriptions = self._extract_element('description')
        self.description = self.descriptions[0]
        # necessary to do an extractor for authors, to present different candidates?
        self.authors = self.article.authors
        self.words = re.split(r' ', self.fulltext)
        self.word_count = len(self.words)
        self.excerpt = ' '.join(self.words if self.word_count < EXCERPT_WORD_COUNT else self.words[:EXCERPT_WORD_COUNT] + ['...'])
        self.published_at = self.article.publish_date.strftime('%Y-%m-%d') if self.article.publish_date else ''
        _url = urlparse(self.returned_url)
        # stuff
        self.url_scheme = _url.scheme
        self.url_domain = _url.netloc
        self.url_path = _url.path
        self.url_fragment = _url.fragment
        self.url_query_string = _url.query
        self.slug = slugify_url(' '.join([self.title, self.url_domain,
                                self.url_fragment]))



    def _extract_element(self, element_name):
        return list(extract_element_from_doc('title', self.doc).values())


    # def _fetch(self, url):
    #     return requests.get(url)

    # def _parse(self):
    #     return htmlparser.fromstring(self.html)

    def _articlize(self):
        # todo, make less redundant
        article = Article(self.fetched_url)
        article.download()
        article.parse()
        return article



from collections import OrderedDict
from importlib import import_module

def extract_element_from_doc(element_name, doc):
    selectors = get_selectors_by_name(element_name)
    candidates = OrderedDict()
    for key, sel in selectors.items():
        selstring = "//%s" % sel
        selval = doc.xpath(selstring)
        if selval and selval[0]:
            candidates[key] = str(selval[0])
    return candidates


def extract_element(element_name, resource): # to be deprecated...?
    return extract_element_from_doc(resource.doc)

def get_selectors_by_name(element_name):
    return import_module('merle.document_extracts.{0}'.format(element_name)).SELECTORS



#!/usr/bin/env python

import sys
from setuptools import setup


install_requires = [
    'newspaper3k>=0.1.7',
    'click>=6.6',
    'jinja2>=2.8',
    'awesome-slugify>=1.6.5'
]


setup(
    name = "merle",
    version='0.0.2.1',
    description='Command-line tool for extracting metadata from URLs',
    long_description='Yada',
    author='Dan Nguyen',
    author_email='dansonguyen@gmail.com',
    url='https://github.com/dannguyen/merle',
    license='MIT',
    classifiers=[
            'Programming Language :: Python :: 3.4',
            'Environment :: Console',
            'Intended Audience :: Developers'
    ],
    packages=['merle'],
    package_data={'jinja templates': 'merle/templates/*.txt'},
    entry_points={
        'console_scripts': ['merle = merle.cli:fetch_metadata']
    },
    install_requires = install_requires
)
